unit Unit1;

{$MODE objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  Menus, BGRABitmap, BGRABitmapTypes, LMessages, Types;

(* http://wiki.freepascal.org/BGRABitmap_tutorial_6 *)
(* http://wiki.lazarus.freepascal.org/Sample_Graphics *)

type
  Six = 0..5;
  Three = 0..2;
  TData = array[Six] of record
    scale, value: integer;
  end;
  
const
  CData: TData = (
    (scale: 100; value: 0),
    (scale: 100; value: 0),
    (scale: 100; value: 0),
    (scale: 100; value: 0),
    (scale: 100; value: 0),
    (scale: 100; value: 0)
  );

type
  { TForm1 }
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure MenuItem2Click(Sender: TObject);
    procedure MenuItem3Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure WMEraseBkgnd(var {%H-}Message: TLMEraseBkgnd); message LM_ERASEBKGND;
  private
    { private declarations }
    background: TBGRABitmap;
    virtualscreen: TBGRABitmap;
    data: array[Three] of TData;
  public
    { public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

function ExePath(): string;
begin
  result := AppendPathDelim(ExtractFilePath(Application.ExeName));
end;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  i, j: integer;
begin
  Randomize;
  
  background := TBGRABitmap.Create(512, 512, BGRAWhite);
  virtualscreen := TBGRABitmap.Create(background.Width, background.Height);

  Timer1.Interval := 500;
  Timer1.Enabled := FALSE;
  
  for i := Low(Three) to High(Three) do
    data[i] := CData;
  
  for j := Low(Six) to High(Six) do
  begin
    data[0][j].value := (3 + (Random(3) - 1)) * (data[0][j].scale div 9);
    data[1][j].value := data[1][j].scale div 2;
    data[2][j].value := (7 + (Random(3) - 1)) * (data[0][j].scale div 9);;
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  background.Free;
  virtualscreen.Free;
end;

procedure TForm1.FormPaint(Sender: TObject);
type
  TPoints = array[Six] of TPointF;
var
  radius: integer;

  procedure FillPointsArray(const aData: TData; var aPoints: TPoints);
  var
    i: integer;
  begin
    for i := Low(Six) to High(Six) do
      aPoints[i] := PointF(
        radius + Cos(i * PI / 3) * (aData[i].value / aData[i].scale) * (3 / 4) * radius,
        radius + Sin(i * PI / 3) * (aData[i].value / aData[i].scale) * (3 / 4) * radius
      );
  end;

var
  points: TPoints;
  pixels: array[Three] of TBGRAPixel;
  i: integer;
  x, y: double;
  textcontent: string;
  textbmp: TBGRABitmap;
  size: TSize;
begin
  pixels[0] := BGRA(255, 0, 0, 255);
  pixels[1] := BGRA(0, 255, 0, 255);
  pixels[2] := BGRA(0, 0, 255, 255);
  
  virtualscreen.PutImage(0, 0, background, dmSet);

  radius := virtualscreen.width div 2;
  
  (* Lignes *)
  
  for i := Low(Six) to High(Six) do
  begin
    x := radius + Cos(i * PI / 3) * (3 / 4) * radius;
    y := radius + Sin(i * PI / 3) * (3 / 4) * radius;
    virtualscreen.DrawLineAntialias(radius, radius, x, y, BGRA(128,128,128,255), 1);
  end;
  
  (* Légende *)
  
  background.FontAntialias := true;
  background.FontHeight := 3 * radius div 32;
  //background.FontStyle := [fsBold];
  background.FontName := 'Courier New';
  
  for i := Low(Six) to High(Six) do
  begin
    x := radius + Cos(i * PI / 3) * (7 / 8) * radius;
    y := radius + Sin(i * PI / 3) * (7 / 8) * radius;
    
    textcontent := Format('%.2d', [i]);
    size := background.TextSize(textcontent);
    textbmp := TBGRABitmap.Create(size.cx + 2, size.cy + 2, BGRAWhite);
    background.CopyPropertiesTo(textbmp);
    textbmp.TextOut(1, 1, textcontent, BGRABlack);
    
    virtualscreen.PutImage(
      Round(x) - textbmp.Width div 2,
      Round(y) - textbmp.Height div 2,
      textbmp,
      dmDrawWithTransparency
    );
    textbmp.Free;
  end;
  
  (* Polygones *)
  
  for i := Low(Three) to High(Three) do
  begin
    FillPointsArray(data[i], points);
    virtualscreen.DrawPolygonAntialias(points, pixels[i], 4);
  end;
  
  virtualscreen.Draw(Canvas, 0, 0);
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  BorderStyle := bsSingle;
  ClientWidth := virtualscreen.width;
  ClientHeight:= virtualscreen.height;
end;

procedure TForm1.MenuItem2Click(Sender: TObject);
begin
  Timer1.Enabled := TRUE;
end;

procedure TForm1.MenuItem3Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  j: integer;
begin
  for j := Low(TData) to High(TData) do
    data[1][j].value := data[1][j].value + (Random(3) - 1) * Random(2);
  Invalidate;
end;

procedure TForm1.WMEraseBkgnd(var Message: TLMEraseBkgnd);
begin
  (* Ne pas effacer le fond *)
end;

end.
